-- ------------------------------------------------
-- SIMULADOR DE TECLADO PS2
-- ------------------------------------------------
--
-- ------------------------------------------------
-- UNIVERSIDAD DE LA REPUBLICA -- FACULTAD DE INGENIERA -- INSTITUTO DE INGENIERA ELECTRICA
--
-- CURSO: Diseo Logico (laboraotiro)
--
-- Javier Rodrguez 
--
-- Version 1.0: 05/10/2010
--
-- ------------------------------------------------
--
-- Circuito que simula las seales DATA y CLOCK que genera un teclado convencional PS2
-- 
-- Entradas:
--				CLK			: Reloj de 50 MHz de la placa del curso
--				SW[3..0]	: Switches 3 a 0 de la placa del curso
--				BUTTON2		: BUTTON2  de la placa del curso
--
-- Salidas:
--				CLOCK		: Seal de reloj de sincronismo simulada.
--				DATA		: Seal de datos PS2 simulada
--
-- ------------------------------------------------
-- FUNCIONAMIENTO
-- ------------------------------------------------
--
--	En la practica se leen el valor de las teclas 0 a 9 del teclado numrico.
--	Aqu para simular que se oprimi una tecal, primero se indica el nmero con
--	SW[3..0] y luego se oprime el botn BUTTON2. El comportamiento de las seales
--	DATA y CLOCK ser tal como si el botn fuera una tecla y el nmero oprimido 
--	de la supuesta tecla, el indicado por SW[3..0].
--	Los valores entre 10 y 15 generan una transmisin de datos de una letra no numrica
--	a los efectos de comprobar que cuando se oprime 1 tecla no numrica el circuito 
--	diseado por los estudiantes funcione correctamente.
--

LIBRARY ieee ;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;



ENTITY SimuladorPS2 is	
	PORT(
		-- Entradas
			Boton		: in std_logic; 					-- Botn 2 de la DE0
			Switches	: in std_logic_vector(3 downto 0); 	-- Entrade de los SW[3..0]
			CLK_placa	: in std_logic; 					-- Seal de CLK de la placa (50MHz)
			Reset		: in std_logic;						-- Reset activo por nivel bajo.
						
		-- Salidas
			DATA_PS2	: out std_logic;
			CLOCK_PS2	: out std_logic

		);
END SimuladorPS2;


ARCHITECTURE FUNCA_TECLADO OF SimuladorPS2 IS

-------------------------------------
-- Signals Principal
-------------------------------------
constant cte_delay	:integer := 57; -- Delay rate:  57 equivale ms o menos a 500ms
constant cte_type	:integer := 10; -- Typematic rate: 10 equivale ms o menos a 91ms.

type	estados_principal is (q_pp_idle,q_send1,q_decido,q_sendF0,q_send2);

signal	qppal,nxqppal	:estados_principal;
signal	Cte,nxCte	:integer range 0 to 63;
signal	CodeM		:std_logic_vector(7 downto 0);			
signal	nxCodeM		:std_logic_vector(7 downto 0);			
signal	Dato_in		:std_logic_vector(7 downto 0);
signal	Start		:std_logic;
signal	Cont_pp		:integer range 0 to 8191;
signal	nxCont_pp	:integer range 0 to 8191;

-------------------------------------
-- Signals Transmisin
-------------------------------------
type	estados_transmision is (q_tr_idle, q_bitarranque, q_data, q_paridad, q_stopbit);

signal	qtr,nxqtr	:estados_transmision;
signal	Dato_out	:std_logic_vector(7 downto 0);			-- Se carga el Codigo_PS2 que luego se va a rotar y enviar.
signal	nxDato_out	:std_logic_vector(7 downto 0);			-- Se carga el Codigo_PS2 que luego se va a rotar y enviar.
signal	Data		:std_logic;
signal	Fin_tr		:std_logic;
signal	Cont_tr		:integer range 0 to 7;
signal	nxCont_tr	:integer range 0 to 7;

-------------------------------------
-- Signals generador de relojes
-------------------------------------
signal 	CLK_cont	:integer range 0 to 2047;
signal	CLK_PS2		:std_logic;
signal	CLK			:std_logic;
signal	Ena_CLK_PS2	:std_logic;

-- Decodificador
signal	num_tecla	:std_logic_vector(3 downto 0);			-- Nmero de "tecla oprimida".
signal	Code_PS2	:std_logic_vector(7 downto 0);			-- Codigo PS2 de la tecla oprimida.

Begin

num_tecla <= Switches;

DATA_PS2 <= Data;
CLOCK_PS2 <= CLK_PS2;


-------------------------------------------------
-- Modulo principal
-------------------------------------------------
ppal_sync:Process(CLK,Reset)

Begin

	if Reset = '0' then
		qppal <= q_pp_idle;
    elsif (falling_edge(CLK)) then
		Case qppal is
			when q_pp_idle =>	Cte <= nxCte;
								CodeM <= nxCodeM;
			when q_send1 =>	Cont_pp <= nxCont_pp;		
			when q_decido =>	Cont_pp <= nxCont_pp;		
								Cte <= nxCte;
			when others =>
		End case;
		
		qppal <= nxqppal;

	end if;
End Process ppal_sync;

ppal_logica:Process(qppal,Code_PS2,CodeM,Cont_pp,Fin_tr,Boton,Cte)
Begin
	nxCte <= 0;
	nxCodeM <= (others => '0');
	nxCont_pp <= 0;
	Dato_in <= (others => '0');
	Start <= '0';
	
	case qppal is
		when q_pp_idle =>
						nxCte <= cte_delay;
						nxCodeM <= Code_PS2;
						
						if Boton = '0' then
							nxqppal <= q_send1;
						else
							nxqppal <= q_pp_idle;
						end if;
		when q_send1 =>
						Start <= '1';
						nxCont_pp <= 0;
						Dato_in <= CodeM;
						
						if Fin_tr = '0' then
							nxqppal <= q_send1;
						else
							nxqppal <= q_decido;
						end if;
		
		when q_decido =>
						nxCont_pp <= Cont_pp + 1;		
						
						if Cont_pp/128 = Cte_delay then		-- Compara los 5 bits ms altos pa no complicar
							nxCte <= Cte_type;				
						else
							nxCte <= Cte;
						end if;
						
						if Boton = '1' then
							nxqppal <= q_sendF0;
						elsif Cont_pp/128 = Cte then
							nxqppal <= q_send1;
						else
							nxqppal <= q_decido;
						end if;
		
		when q_sendF0 =>
						Start <= '1';
						Dato_in <= "11110000";
						
						if Fin_tr = '0' then
							nxqppal <= q_sendF0;
						else
							nxqppal <= q_send2;
						end if;

		when q_send2 =>
						Start <= '1';
						Dato_in <= CodeM;
						
						if Fin_tr = '0' then
							nxqppal <= q_send2;
						else
							nxqppal <= q_pp_idle;
						end if;
		when others =>
						nxqppal <= q_pp_idle;
	End case;
						
End Process ppal_logica;

-------------------------------------------------
-- Modulo transmisor
-------------------------------------------------

tr_sync:Process(CLK, Reset)
Begin
	if Reset = '0' then
		qtr <= q_tr_idle;
    elsif falling_edge(CLK) then

		case qtr is
			when q_tr_idle =>		Dato_out <= nxDato_out;
			when q_bitarranque =>	Cont_tr <= nxCont_tr;
			when q_data =>			Dato_out <= nxDato_out;
									Cont_tr <= nxCont_tr;
			when others =>
		End Case;

		qtr <= nxqtr;		

	End if;

End process tr_sync;

tr_logica:Process(qtr, Start,Cont_tr, Dato_in, Dato_out)
Begin
	
	nxCont_tr <= 0;
	nxDato_out <= (others => '0');
	Fin_tr <= '0';
	
	case qtr is
		
		when q_tr_idle =>
						Ena_CLK_PS2 <= '0';
						Data <= '1';
						nxDato_out <= Dato_in;
						
						if (Start = '0') then
							nxqtr <= q_tr_idle;
						else
							nxqtr <= q_bitarranque;
						end if;
							
		when q_bitarranque =>
						Data <= '0';
						Ena_CLK_PS2 <= '1';
						nxCont_tr <= 0;
						nxqtr <= q_data;
	
		when q_data =>
						Data <= Dato_out(0);
						Ena_CLK_PS2 <= '1';
						nxDato_out <= Dato_out(0) & Dato_out(7 downto 1);
						nxCont_tr <= Cont_tr + 1;
							
						if (Cont_tr = 7) then
							nxqtr <= q_paridad;
						else
							nxqtr <= q_data;
						end if;
					
		when q_paridad =>
						Data <= not (Dato_out(7) xor Dato_out(6) xor Dato_out(5) xor Dato_out(4)xor 
									Dato_out(3) xor Dato_out(2) xor Dato_out(1) xor Dato_out(0));
						Ena_CLK_PS2 <= '1';
						nxqtr <= q_stopbit;
						
		when q_stopbit =>
						Data <= '1';
						Ena_CLK_PS2 <= '1';
						Fin_tr <= '1';
						nxqtr <= q_tr_idle;
						
		when others =>
						Data <= '1';
						Ena_CLK_PS2 <= '0';
						nxqtr <= q_tr_idle;
	End case;
		
End Process tr_logica;

-------------------------------------
-- Modulo generador de relojes
-------------------------------------

relojes: Process(CLK_placa,Reset)
Begin
	if Reset = '0' then
		CLK_cont <= 0;			
		CLK <= '1';
		CLK_PS2 <= '1';
	elsif (falling_edge(CLK_placa)) then
		if CLK_cont < 1666 then					-- Los 1666 es para rondar los 15 KHz (50MHz /1667 / 2)
			CLK_cont <= CLK_cont + 1;
		else 
			CLK_cont <= 0;
		end if;
		
		if CLK_cont = 1200 then					-- Los 1200 es para retrasar el CLK_PS2 respecto a CLK
			CLK <= not CLK;
		end if;
		if Ena_CLK_PS2 = '1' and CLK_cont = 0 then  
			CLK_PS2 <= CLK;
		end if;		
	
	end if;
End Process relojes;   

-------------------------------------
-- Modulo decodificador
-------------------------------------
decodificador_PS2: Process (num_tecla)

Begin
		
	Case num_tecla is
	
		when "0000" =>	Code_PS2 <= "01110000"; -- 70h
		when "0001" =>	Code_PS2 <= "01101001";	-- 69h
		when "0010" =>	Code_PS2 <= "01110010";	-- 72h
		when "0011" =>	Code_PS2 <= "01111010";	-- 7Ah
		when "0100" =>	Code_PS2 <= "01101011";	-- 6Bh
		when "0101" =>	Code_PS2 <= "01110011"; -- 73h
		when "0110" =>	Code_PS2 <= "01110100";	-- 74h
		when "0111" =>	Code_PS2 <= "01101100";	-- 6Ch
		when "1000" =>	Code_PS2 <= "01110101";	-- 75h
		when "1001" =>	Code_PS2 <= "01111101";	-- 7Dh
		when "1010" =>	Code_PS2 <= "10001010";	-- 8Ah
		when "1011" =>	Code_PS2 <= "10001011";	-- 8Bh
		when "1100" =>	Code_PS2 <= "10001100";	-- 8Ch
		when "1101" =>	Code_PS2 <= "10001101";	-- 8Dh
		when "1110" =>	Code_PS2 <= "10001110";	-- 8Eh
		when others =>	Code_PS2 <= "01010000";	-- 50h
		
	end case;

End Process decodificador_PS2;


END FUNCA_TECLADO;
